<?php
// +----------------------------------------------------------------------
// | ThinkPHP [ WE CAN DO IT JUST THINK ]
// +----------------------------------------------------------------------
// | Copyright (c) 2006~2019 http://thinkphp.cn All rights reserved.
// +----------------------------------------------------------------------
// | Licensed ( http://www.apache.org/licenses/LICENSE-2.0 )
// +----------------------------------------------------------------------
// | Author: liu21st <liu21st@gmail.com>
// +----------------------------------------------------------------------
declare (strict_types = 1);

namespace think;

use app\common\model\AttachMain;

/**
 * 请求管理类
 */
class Resource
{
    private $link;
    private $web_site;
    private $html;

    public function __construct($web_site, $link = '')
    {
        $this->web_site = $web_site;
        $this->link     = $link;
    }

    private function getHtmlInfo($convertToUTF8 = false)
    {
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_TIMEOUT, 30);
        curl_setopt($curl, CURLOPT_URL, $this->link);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt($curl, CURLOPT_HEADER, false);
        curl_setopt($curl, CURLOPT_HTTPHEADER, [
            'Accept: text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3',
            'Accept-Encoding: gzip, deflate, br',
            'Accept-Language: zh-CN,zh;q=0.9',
            'Cache-Control: max-age=0',
            'Connection: keep-alive',
            'Host: ' . trim(str_replace(['http://', 'https://'], '', $this->web_site['url']), '/'),
            'Upgrade-Insecure-Requests: 1',
            'User-Agent: Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/75.0.3770.142 Safari/537.36',
        ]);
        curl_setopt($curl, CURLOPT_ENCODING, 'gzip');
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        $html = curl_exec($curl);
        curl_close($curl);
        if ($convertToUTF8 == true) {
            $html = iconv('GBK', 'UTF-8', $html);
        }
        $this->html = $html;
        preg_match('/<title>(.*?)<\/title>/', $html, $title);
        return ['title' => $title[1] ?? '获取失败', 'web_site' => $this->web_site, 'link' => $this->link];

    }

    private function get_cache($site_code = '', $site_code_type = '')
    {
        $map = [
            ['site_id', '=', $this->web_site['site_id']],
            ['site_code', '=', $site_code],
            ['status', '>', 1],
        ];
        if (!empty($site_code_type)) {
            $map[] = ['site_code_type', '=', $site_code_type];
        }
        $query = AttachMain::where($map)->select();
        if (!$query->isEmpty()) {
            $download = [];
            foreach ($query as $attach) {
                $download[$attach['button_name']] = url('index/download/cache', ['attach_id' => $attach['attach_id']]);
            }
            if (!empty($download)) {
                return $download;
            }
        }
        return false;
    }

    public function getHtml()
    {
        return $this->html;
    }

    public function get_51yuansu_com()
    {
        preg_match('/\/(\w+).html/', $this->link, $site_code);
        $download = $this->get_cache($site_code['1']);
        if ($download === false) {
            return false;
        }
        return ['info' => $this->getHtmlInfo(), 'download' => $download];
    }

    public function get_ooopic_com()
    {
        preg_match('/\/pic_([0-9]+).html/', $this->link, $site_code);
        $download = $this->get_cache($site_code['1']);
        if ($download === false) {
            return false;
        }
        return ['info' => $this->getHtmlInfo(true), 'download' => $download];
    }

    public function get_58pic_com()
    {
        preg_match('/\/([0-9]+).html/', $this->link, $site_code);
        $download = $this->get_cache($site_code['1']);
        if ($download === false) {
            return false;
        }
        return ['info' => $this->getHtmlInfo(true), 'download' => $download];
    }

    public function get_nipic_com()
    {
        preg_match('/\/([0-9]+).html/', $this->link, $site_code);
        $download = $this->get_cache($site_code['1']);
        if ($download === false) {
            return false;
        }
        return ['info' => $this->getHtmlInfo(), 'download' => $download];
    }

    public function get_90sheji_com()
    {
        preg_match('/\/([0-9]+).html/', $this->link, $site_code);
        $download = $this->get_cache($site_code['1']);
        if ($download === false) {
            return false;
        }
        return ['info' => $this->getHtmlInfo(true), 'download' => $download];
    }

    public function get_588ku_com()
    {
        $site_code      = [];
        $site_code_type = '';
        //免扣元素 背景图库 设计模板 摄影图库 艺术字 UI设计 商用插画 办公文档 视频 字体库
        foreach (['ycpng', 'ycbeijing', 'moban', 'sheyingtu', 'ycwordart', 'uiweb', 'ichahua', 'office', 'video', 'font', 'ycaudio'] as $type) {
            preg_match('/' . $type . '\/([0-9]+).html/', $this->link, $match);
            if (!empty($match[1])) {
                $site_code_type = $type;
                $site_code      = $match;
                break;
            }
        }
        $download = $this->get_cache($site_code['1'], $site_code_type);
        if ($download === false) {
            return false;
        }
        return ['info' => $this->getHtmlInfo(), 'download' => $download];
    }

    public function get_ibaotu_com()
    {
        preg_match('/\/([0-9]+).html/', $this->link, $site_code);
        $download = $this->get_cache($site_code['1']);
        if ($download === false) {
            return false;
        }
        return ['info' => $this->getHtmlInfo(), 'download' => $download];
    }

    public function get_699pic_com()
    {
        preg_match('/\/video-([0-9]+).html/', $this->link, $site_code); //视频
        if (!empty($site_code['1'])) {
            $download = $this->get_cache($site_code['1'], 'video');
            if ($download === false) {
                return false;
            }
            return ['info' => $this->getHtmlInfo(), 'download' => $download];
        }
        preg_match('/\/media-([0-9]+).html/', $this->link, $site_code); //视频
        if (!empty($site_code['1'])) {
            $download = $this->get_cache($site_code['1'], 'media');
            if ($download === false) {
                return false;
            }
            return ['info' => $this->getHtmlInfo(), 'download' => $download];
        }
        preg_match('/\/music-([0-9]+).html/', $this->link, $site_code); //视频
        if (!empty($site_code['1'])) {
            $download = $this->get_cache($site_code['1'], 'music');
            if ($download === false) {
                return false;
            }
            return ['info' => $this->getHtmlInfo(), 'download' => $download];
        }
        preg_match('/\/font-([0-9]+).html/', $this->link, $site_code); //字体
        if (!empty($site_code['1'])) {
            $download = $this->get_cache($site_code['1'], 'font');
            if ($download === false) {
                return false;
            }
            return ['info' => $this->getHtmlInfo(), 'download' => $download];
        }
        return false;
    }

    public function get_download_csdn_net()
    {
        preg_match('/download\/.*?\/([0-9]+)/', $this->link, $site_code);
        $download = $this->get_cache($site_code['1']);
        if ($download === false) {
            return false;
        }
        return ['info' => $this->getHtmlInfo(), 'download' => $download];
    }

    public function get_docer_com()
    {
        preg_match('/webmall\/preview\/([0-9]+).*?/', $this->link, $site_code);
        $download = $this->get_cache($site_code['1']);
        if ($download === false) {
            return false;
        }
        return ['info' => $this->getHtmlInfo(), 'download' => $download];
    }

    public function get_wenku_baidu_com()
    {
        preg_match('/view\/(\w+)/', $this->link, $site_code);
        $download = $this->get_cache($site_code['1']);
        if ($download === false) {
            return false;
        }
        return ['info' => $this->getHtmlInfo(true), 'download' => $download];
    }

    public function get_17sucai_com()
    {
        preg_match('/\/([0-9]+).html/', $this->link, $site_code);
        $download = $this->get_cache($site_code['1']);
        if ($download === false) {
            return false;
        }
        return ['info' => $this->getHtmlInfo(), 'download' => $download];
    }

    public function get_tukuppt_com()
    {
        preg_match('/\/(\w+).html/', $this->link, $site_code);
        $download = $this->get_cache($site_code['1']);
        if ($download === false) {
            return false;
        }
        return ['info' => $this->getHtmlInfo(), 'download' => $download];
    }

    public function get_92sucai_com()
    {
        preg_match('/\/([0-9]+).html/', $this->link, $site_code);
        $download = $this->get_cache($site_code['1']);
        if ($download === false) {
            return false;
        }
        return ['info' => $this->getHtmlInfo(), 'download' => $download];
    }

    public function get_yanj_cn()
    {
        preg_match('/\/([0-9]+).html/', $this->link, $site_code);
        $download = $this->get_cache($site_code['1']);
        if ($download === false) {
            return false;
        }
        return ['info' => $this->getHtmlInfo(), 'download' => $download];
    }

    public function get_pic_netbian_com()
    {
        preg_match('/\/([0-9]+).html/', $this->link, $site_code);
        $download = $this->get_cache($site_code['1']);
        if ($download === false) {
            return false;
        }
        return ['info' => $this->getHtmlInfo(true), 'download' => $download];
    }

    public function get_huiyi8_com()
    {
        preg_match('/\/([0-9]+).html/', $this->link, $site_code);
        $download = $this->get_cache($site_code['1']);
        if ($download === false) {
            return false;
        }
        return ['info' => $this->getHtmlInfo(), 'download' => $download];
    }

    public function get_88tph_com()
    {
        preg_match('/\/([0-9]+).html/', $this->link, $site_code);
        $download = $this->get_cache($site_code['1']);
        if ($download === false) {
            return false;
        }
        return ['info' => $this->getHtmlInfo(), 'download' => $download];
    }

    public function get_51miz_com()
    {
        preg_match('/\/([0-9]+).html/', $this->link, $site_code);
        $download = $this->get_cache($site_code['1']);
        if ($download === false) {
            return false;
        }
        return ['info' => $this->getHtmlInfo(), 'download' => $download];
    }

    public function get_16pic_com()
    {
        preg_match('/\/pic_([0-9]+).html/', $this->link, $site_code);
        $download = $this->get_cache($site_code['1']);
        if ($download === false) {
            return false;
        }
        return ['info' => $this->getHtmlInfo(), 'download' => $download];
    }

    public function get_125pic_com()
    {
        preg_match('/\/([0-9]+)/', $this->link, $site_code);
        $download = $this->get_cache($site_code['1']);
        if ($download === false) {
            return false;
        }
        return ['info' => $this->getHtmlInfo(), 'download' => $download];
    }

    public function get_ppt_dwuva_com()
    {
        preg_match('/\/([A-Za-z0-9]+).html/', $this->link, $site_code);
        $download = $this->get_cache($site_code['1']);
        if ($download === false) {
            return false;
        }
        return ['info' => $this->getHtmlInfo(), 'download' => $download];
    }

    public function get_ppt118_com()
    {
        preg_match('/\/([A-Za-z0-9]+).html/', $this->link, $site_code);
        $download = $this->get_cache($site_code['1']);
        if ($download === false) {
            return false;
        }
        return ['info' => $this->getHtmlInfo(), 'download' => $download];
    }

    public function get_yipic_cn()
    {
        preg_match('/\/([0-9]+).html/', $this->link, $site_code);
        $download = $this->get_cache($site_code['1']);
        if ($download === false) {
            return false;
        }
        return ['info' => $this->getHtmlInfo(), 'download' => $download];
    }

    public function get_photophoto_cn()
    {
        preg_match('/\/([0-9]+).html/', $this->link, $site_code);
        $download = $this->get_cache($site_code['1']);
        if ($download === false) {
            return false;
        }
        return ['info' => $this->getHtmlInfo(), 'download' => $download];
    }

    public function get_669pic_com()
    {
        preg_match('/\/([0-9]+).html/', $this->link, $site_code);
        $download = $this->get_cache($site_code['1']);
        if ($download === false) {
            return false;
        }
        return ['info' => $this->getHtmlInfo(), 'download' => $download];
    }

    public function get_kuaipng_com()
    {
        preg_match('/\/([0-9]+).html/', $this->link, $site_code);
        $download = $this->get_cache($site_code['1']);
        if ($download === false) {
            return false;
        }
        return ['info' => $this->getHtmlInfo(), 'download' => $download];
    }

    public function get_7msj_com()
    {
        preg_match('/\/forum.php\?mod=viewthread&tid=([0-9]+)/', $this->link, $site_code);
        $download = $this->get_cache($site_code['1']);
        if ($download === false) {
            return false;
        }
        return ['info' => $this->getHtmlInfo(true), 'download' => $download];
    }

    public function get_jq22_com()
    {
        preg_match('/\/jquery-info([0-9]+)/', $this->link, $site_code);
        $download = $this->get_cache($site_code['1']);
        if ($download === false) {
            return false;
        }
        return ['info' => $this->getHtmlInfo(), 'download' => $download];
    }

    public function get_huke88_com()
    {
        preg_match('/\/([0-9]+).html/', $this->link, $site_code);
        $download = $this->get_cache($site_code['1']);
        if ($download === false) {
            return false;
        }
        return ['info' => $this->getHtmlInfo(), 'download' => $download];
    }
}
